const fs = require('fs')
const fetch = require('node-fetch')
const input = require('./28276188.json')

// filter out measeurements that have obvious errors
const withoutMsmErrors = input.filter(msm => {
  return !(msm.result.length === 1 && msm.result[0].hasOwnProperty('error'))
})

// filter out measurements that have NO msm error, but its result holds entries from hops
// that reported errors - these are also considered failed as these will NOT reach the 
// destination node, and because of this the penultimate hop in these results will not be 
// teredo tunnel providers so we don't care about them
const hopHasErr = msm => msm.result.some(hop => hop.result.some(hops => hops.hasOwnProperty('err')))

const withoutHopErrors = withoutMsmErrors.filter(msm => {
  // if msm has only 1 hop in its result, then it's safe to assume that it is 
  // not a valuable measurement ie. never reached the destination node, so 
  // do not attempt to look into it further
  if (msm.result.length === 1) {
    return false
  }
  // otherwise see if msm result hops report any errors 
  // keep those msms that have no hops with errors
  return !hopHasErr(msm)
})

// take msm destination address from first result's metadata - 
// since all results are from the same measurement, it's safe 
// to assume that the dst_addr will be the same for all of them
const msmDestination = input[0].dst_addr

// get source addresses & the address of the penultimate hop
// from error-free measurements
const msmSourceAndPenultimate = withoutHopErrors
  .map(x => {
    // some results reported from hops can still have timeouts
    // so have to remove these from the hop's result array
    const penultimateResults = x.result[x.result.length-2].result
      .filter(x => !x.hasOwnProperty('x'))
    return {
      src: x.src_addr,
      probeId: x.prb_id,
      ph: penultimateResults.length > 0
        ? penultimateResults[0].from
        : null,
    }
  })
  .filter(x => x.ph !== null)


const tree = {
  name: msmDestination,
  children: [],
}

const asURL = 'https://stat.ripe.net/data/prefix-overview/data.json?max_related=0&resource='
const getAs = async prefix => {
  let asnum = null 
  try {
    const requestURL = asURL + prefix
    const response = await fetch(requestURL)
    const data = await response.json()
    asnum = data.data.asns[0].asn
  } catch(e) {
  }
  return asnum
}

const hostnameURL = 'https://stat.ripe.net/data/reverse-dns-ip/data.json?resource='
const getHostname = async prefix => {
  let hostname = null
  try {
    const requestURL = hostnameURL + prefix
    const response = await fetch(requestURL)
    const data = await response.json()
    hostname = data.data.result[0]
  } catch(e) {
  }
  return hostname
}


const geolocURL = 'https://stat.ripe.net/data/maxmind-geo-lite/data.json?resource='
const getGeoloc = async prefix => {
  let coords = null
  try {
    const requestURL = geolocURL + prefix
    const response = await fetch(requestURL)
    const data = await response.json()
    const location = data.data.located_resources[0].locations[0]
    coords = [location.latitude, location.longitude]
  } catch(e) {
  }
  return coords 
}

msmSourceAndPenultimate.forEach(async entry => {
  let phNode = tree.children.find(x => x.name === entry.ph)
  if (!phNode) {
    tree.children.push({name: entry.ph, children: []})
    phNode = tree.children.find(x => x.name === entry.ph)
  }
  phNode.children.push({ name: entry.src, location: await getGeoloc(entry.src), probeId: entry.probeId}) 
})


async function enrichHopData() {
  tree.location = await getGeoloc(tree.name)
  tree.hostname = await getHostname(tree.name)
  tree.as = await getAs(tree.name)
  const newPenultimates = await Promise.all(tree.children.map(async c => {
    c.as = await getAs(c.name)
    c.hostname = await getHostname(c.name)
    c.location = await getGeoloc(c.name)
    return c
  }))
  tree.children = newPenultimates
}

enrichHopData()
  .then(() => {
    // create msmSourceAndPenultimate JSON
    let msmData= JSON.stringify(msmSourceAndPenultimate);
    fs.writeFileSync('msmSourceAndPenultimate.json', msmData);

    let treeData = JSON.stringify(tree)
    fs.writeFileSync('treeData.json', treeData);

    console.log('All done.')
  })
